import { AppConfigure } from './AppConfigure';
import { IContainer, LoadType, MapSet, Factory, Token, IContainerBuilder } from '@ts-ioc/core';
import { IApplicationBuilder, CustomRegister, AnyApplicationBuilder } from './IApplicationBuilder';
import { ModuleBuilder, ModuleEnv, InjectedModule, IModuleBuilder, ModuleConfig } from '../modules';
import { ContainerPool, Events, IEvents } from '../utils';
import { Runnable } from '../runnable';
export declare enum ApplicationEvents {
    onRootContainerCreated = "onRootContainerCreated",
    onRootContainerInited = "onRooConatianerInited"
}
/**
 * application builder.
 *
 * @export
 * @class Default ApplicationBuilder
 * @extends {ModuleBuilder}
 * @template T
 */
export declare class DefaultApplicationBuilder<T> extends ModuleBuilder<T> implements IApplicationBuilder<T>, IEvents {
    baseURL?: string;
    private globalConfig;
    protected globalModules: LoadType[];
    protected customRegs: CustomRegister<T>[];
    protected beforeInitPds: MapSet<Token<any>, any>;
    protected afterInitPds: MapSet<Token<any>, any>;
    protected configs: (string | AppConfigure)[];
    inited: boolean;
    events: Events;
    constructor(baseURL?: string);
    protected initEvents(): void;
    static create(baseURL?: string): AnyApplicationBuilder;
    on(name: string, event: (...args: any[]) => void): this;
    off(name: string, event?: (...args: any[]) => void): this;
    emit(name: string, ...args: any[]): void;
    getPools(): ContainerPool;
    protected createContainerBuilder(): IContainerBuilder;
    /**
     * use configuration.
     *
     * @param {(string | AppConfigure)} [config]
     * @returns {this} global config for this application.
     * @memberof Bootstrap
     */
    useConfiguration(config?: string | AppConfigure): this;
    protected loadConfig(container: IContainer, src: string): Promise<AppConfigure>;
    /**
     * use module as global Depdences module.
     *
     * @param {...LoadType[]} modules
     * @returns {this}
     * @memberof PlatformServer
     */
    use(...modules: LoadType[]): this;
    /**
     * bind provider
     *
     * @template T
     * @param {Token<T>} provide
     * @param {Token<T> | Factory<T>} provider
     * @param {boolean} [beforRootInit]
     * @returns {this}
     * @memberof IContainer
     */
    provider(provide: Token<any>, provider: Token<any> | Factory<any>, beforRootInit?: boolean): this;
    protected load(token: Token<T> | AppConfigure, env?: ModuleEnv): Promise<InjectedModule<T>>;
    build(token: Token<T> | AppConfigure, env?: ModuleEnv, data?: any): Promise<T>;
    bootstrap(token: Token<T> | AppConfigure, env?: ModuleEnv, data?: any): Promise<Runnable<T>>;
    /**
     * get module builder
     *
     * @param {(Token<T> | ModuleConfig<T>)} token
     * @param {ModuleEnv} [env]
     * @returns {IModuleBuilder<T>}
     * @memberof IApplicationBuilder
     */
    getBuilderByConfig(token: Token<T> | ModuleConfig<T>, env?: ModuleEnv): Promise<IModuleBuilder<T>>;
    getBuilder(injmdl: InjectedModule<T>): IModuleBuilder<T>;
    protected getDefaultBuilder(container: IContainer): IModuleBuilder<any>;
    protected getGlobalConfig(container: IContainer): Promise<AppConfigure>;
    protected createDefaultContainer(): IContainer;
    protected initRootContainer(container?: IContainer): Promise<void>;
    /**
     * register ioc exts
     *
     * @protected
     * @param {IContainer} container
     * @param {AppConfigure} config
     * @returns {Promise<IContainer>}
     * @memberof ApplicationBuilder
     */
    protected registerExts(container: IContainer, config: AppConfigure): Promise<IContainer>;
    protected bindAppConfig(config: AppConfigure): AppConfigure;
    protected getDefaultConfig(container: IContainer): Promise<AppConfigure>;
    static classAnnations: any;
}
