"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var AppConfigure_1 = require("./AppConfigure");
var core_1 = require("@ts-ioc/core");
var modules_1 = require("../modules");
var utils_1 = require("../utils");
var BootModule_1 = require("../BootModule");
var ApplicationEvents;
(function (ApplicationEvents) {
    ApplicationEvents["onRootContainerCreated"] = "onRootContainerCreated";
    ApplicationEvents["onRootContainerInited"] = "onRooConatianerInited";
})(ApplicationEvents = exports.ApplicationEvents || (exports.ApplicationEvents = {}));
/**
 * application builder.
 *
 * @export
 * @class Default ApplicationBuilder
 * @extends {ModuleBuilder}
 * @template T
 */
var DefaultApplicationBuilder = /** @class */ (function (_super) {
    tslib_1.__extends(DefaultApplicationBuilder, _super);
    function DefaultApplicationBuilder(baseURL) {
        var _this = _super.call(this) || this;
        _this.baseURL = baseURL;
        _this.inited = false;
        _this.customRegs = [];
        _this.globalModules = [];
        _this.configs = [];
        _this.beforeInitPds = new core_1.MapSet();
        _this.afterInitPds = new core_1.MapSet();
        _this.events = new utils_1.Events();
        _this.initEvents();
        return _this;
    }
    DefaultApplicationBuilder.prototype.initEvents = function () {
        var _this = this;
        this.events.on('onRooConatianerInited', function (container) {
            _this.afterInitPds.forEach(function (val, key) {
                container.bindProvider(key, val);
            });
        });
    };
    DefaultApplicationBuilder.create = function (baseURL) {
        return new DefaultApplicationBuilder(baseURL);
    };
    DefaultApplicationBuilder.prototype.on = function (name, event) {
        this.events.on(name, event);
        return this;
    };
    DefaultApplicationBuilder.prototype.off = function (name, event) {
        this.events.off(name, event);
        return this;
    };
    DefaultApplicationBuilder.prototype.emit = function (name) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        var _a;
        (_a = this.events).emit.apply(_a, [name].concat(args));
    };
    DefaultApplicationBuilder.prototype.getPools = function () {
        if (!this.pools) {
            this.pools = new utils_1.ContainerPool(this.createContainerBuilder());
            this.createDefaultContainer();
        }
        return this.pools;
    };
    DefaultApplicationBuilder.prototype.createContainerBuilder = function () {
        return new core_1.DefaultContainerBuilder();
    };
    /**
     * use configuration.
     *
     * @param {(string | AppConfigure)} [config]
     * @returns {this} global config for this application.
     * @memberof Bootstrap
     */
    DefaultApplicationBuilder.prototype.useConfiguration = function (config) {
        if (core_1.isUndefined(config)) {
            config = '';
        }
        // clean cached config.
        this.globalConfig = null;
        var idx = this.configs.indexOf(config);
        if (idx >= 0) {
            this.configs.splice(idx, 1);
        }
        this.configs.push(config);
        return this;
    };
    DefaultApplicationBuilder.prototype.loadConfig = function (container, src) {
        if (container.has(AppConfigure_1.AppConfigureLoaderToken)) {
            var loader = container.resolve(AppConfigure_1.AppConfigureLoaderToken, { baseURL: this.baseURL, container: container });
            return loader.load(src);
        }
        else if (src) {
            var builder = container.getBuilder();
            return builder.loader.load([src])
                .then(function (rs) {
                return rs.length ? rs[0] : null;
            });
        }
        else {
            return Promise.resolve(null);
        }
    };
    /**
     * use module as global Depdences module.
     *
     * @param {...LoadType[]} modules
     * @returns {this}
     * @memberof PlatformServer
     */
    DefaultApplicationBuilder.prototype.use = function () {
        var modules = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            modules[_i] = arguments[_i];
        }
        this.globalModules = this.globalModules.concat(modules);
        this.inited = false;
        return this;
    };
    /**
     * bind provider
     *
     * @template T
     * @param {Token<T>} provide
     * @param {Token<T> | Factory<T>} provider
     * @param {boolean} [beforRootInit]
     * @returns {this}
     * @memberof IContainer
     */
    DefaultApplicationBuilder.prototype.provider = function (provide, provider, beforRootInit) {
        if (beforRootInit) {
            this.beforeInitPds.set(provide, provider);
        }
        else {
            this.afterInitPds.set(provide, provider);
        }
        return this;
    };
    DefaultApplicationBuilder.prototype.load = function (token, env) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.initRootContainer()];
                    case 1:
                        _a.sent();
                        return [2 /*return*/, _super.prototype.load.call(this, token, env)];
                }
            });
        });
    };
    DefaultApplicationBuilder.prototype.build = function (token, env, data) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var injmdl, builder;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.load(token, env)];
                    case 1:
                        injmdl = _a.sent();
                        builder = this.getBuilder(injmdl);
                        return [4 /*yield*/, builder.build(token, injmdl, data)];
                    case 2: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    DefaultApplicationBuilder.prototype.bootstrap = function (token, env, data) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var injmdl, builder;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.load(token, env)];
                    case 1:
                        injmdl = _a.sent();
                        builder = this.getBuilder(injmdl);
                        return [4 /*yield*/, builder.bootstrap(token, injmdl, data)];
                    case 2: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * get module builder
     *
     * @param {(Token<T> | ModuleConfig<T>)} token
     * @param {ModuleEnv} [env]
     * @returns {IModuleBuilder<T>}
     * @memberof IApplicationBuilder
     */
    DefaultApplicationBuilder.prototype.getBuilderByConfig = function (token, env) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var injmdl;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.load(token, env)];
                    case 1:
                        injmdl = _a.sent();
                        return [2 /*return*/, this.getBuilder(injmdl)];
                }
            });
        });
    };
    DefaultApplicationBuilder.prototype.getBuilder = function (injmdl) {
        var cfg = injmdl.config;
        var container = injmdl.container;
        var builder;
        if (cfg) {
            if (core_1.isClass(cfg.builder)) {
                if (!container.has(cfg.builder)) {
                    container.register(cfg.builder);
                }
            }
            if (core_1.isToken(cfg.builder)) {
                builder = container.resolve(cfg.builder);
            }
            else if (cfg.builder instanceof modules_1.ModuleBuilder) {
                builder = cfg.builder;
            }
        }
        var tko = injmdl.token;
        if (!builder && tko) {
            builder = container.getRefService(modules_1.InjectModuleBuilderToken, tko, modules_1.DefaultModuleBuilderToken);
        }
        if (!builder) {
            builder = this.getDefaultBuilder(container);
        }
        return builder || this;
    };
    DefaultApplicationBuilder.prototype.getDefaultBuilder = function (container) {
        return container.resolve(modules_1.ModuleBuilderToken);
    };
    DefaultApplicationBuilder.prototype.getGlobalConfig = function (container) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var globCfg_1, exts;
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!!this.globalConfig) return [3 /*break*/, 3];
                        return [4 /*yield*/, this.getDefaultConfig(container)];
                    case 1:
                        globCfg_1 = _a.sent();
                        if (this.configs.length < 1) {
                            this.configs.push(''); // load default loader config.
                        }
                        return [4 /*yield*/, Promise.all(this.configs.map(function (cfg) {
                                if (core_1.isString(cfg)) {
                                    return _this.loadConfig(container, cfg);
                                }
                                else {
                                    return cfg;
                                }
                            }))];
                    case 2:
                        exts = _a.sent();
                        exts.forEach(function (exCfg) {
                            if (exCfg) {
                                core_1.lang.assign(globCfg_1, exCfg);
                            }
                        });
                        this.globalConfig = globCfg_1;
                        _a.label = 3;
                    case 3: return [2 /*return*/, this.globalConfig];
                }
            });
        });
    };
    DefaultApplicationBuilder.prototype.createDefaultContainer = function () {
        var _this = this;
        var container = this.pools.getDefault();
        container.register(BootModule_1.BootModule);
        var chain = container.getBuilder().getInjectorChain(container);
        chain.first(container.resolve(modules_1.DIModuleInjectorToken));
        container.bindProvider(utils_1.ContainerPoolToken, function () { return _this.getPools(); });
        this.beforeInitPds.forEach(function (val, key) {
            container.bindProvider(key, val);
        });
        this.events.emit(ApplicationEvents.onRootContainerCreated, container);
        return container;
    };
    DefaultApplicationBuilder.prototype.initRootContainer = function (container) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var globCfg;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.inited) {
                            return [2 /*return*/];
                        }
                        container = container || this.getPools().getDefault();
                        return [4 /*yield*/, this.getGlobalConfig(container)];
                    case 1:
                        globCfg = _a.sent();
                        return [4 /*yield*/, this.registerExts(container, globCfg)];
                    case 2:
                        _a.sent();
                        this.bindAppConfig(globCfg);
                        container.bindProvider(AppConfigure_1.AppConfigureToken, globCfg);
                        this.inited = true;
                        this.events.emit(ApplicationEvents.onRootContainerInited, container);
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * register ioc exts
     *
     * @protected
     * @param {IContainer} container
     * @param {AppConfigure} config
     * @returns {Promise<IContainer>}
     * @memberof ApplicationBuilder
     */
    DefaultApplicationBuilder.prototype.registerExts = function (container, config) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var usedModules;
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this.globalModules.length) return [3 /*break*/, 2];
                        usedModules = this.globalModules;
                        return [4 /*yield*/, container.loadModule.apply(container, usedModules)];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2:
                        if (!this.customRegs.length) return [3 /*break*/, 4];
                        return [4 /*yield*/, Promise.all(this.customRegs.map(function (cs) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                                var tokens;
                                return tslib_1.__generator(this, function (_a) {
                                    switch (_a.label) {
                                        case 0: return [4 /*yield*/, cs(container, config, this)];
                                        case 1:
                                            tokens = _a.sent();
                                            return [2 /*return*/, tokens];
                                    }
                                });
                            }); }))];
                    case 3:
                        _a.sent();
                        _a.label = 4;
                    case 4: return [2 /*return*/, container];
                }
            });
        });
    };
    DefaultApplicationBuilder.prototype.bindAppConfig = function (config) {
        if (this.baseURL) {
            config.baseURL = this.baseURL;
        }
        return config;
    };
    DefaultApplicationBuilder.prototype.getDefaultConfig = function (container) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                if (container.has(AppConfigure_1.DefaultConfigureToken)) {
                    return [2 /*return*/, container.resolve(AppConfigure_1.DefaultConfigureToken)];
                }
                else {
                    return [2 /*return*/, {}];
                }
                return [2 /*return*/];
            });
        });
    };
    DefaultApplicationBuilder.classAnnations = { "name": "DefaultApplicationBuilder", "params": { "constructor": ["baseURL"], "initEvents": [], "create": ["baseURL"], "on": ["name", "event"], "off": ["name", "event"], "emit": ["name", "args"], "getPools": [], "createContainerBuilder": [], "useConfiguration": ["config"], "loadConfig": ["container", "src"], "use": ["modules"], "provider": ["provide", "provider", "beforRootInit"], "load": ["token", "env"], "build": ["token", "env", "data"], "bootstrap": ["token", "env", "data"], "getBuilderByConfig": ["token", "env"], "getBuilder": ["injmdl"], "getDefaultBuilder": ["container"], "getGlobalConfig": ["container"], "createDefaultContainer": [], "initRootContainer": ["container"], "registerExts": ["container", "config"], "bindAppConfig": ["config"], "getDefaultConfig": ["container"] } };
    return DefaultApplicationBuilder;
}(modules_1.ModuleBuilder));
exports.DefaultApplicationBuilder = DefaultApplicationBuilder;

//# sourceMappingURL=../sourcemaps/boot/ApplicationBuilder.js.map

//# sourceMappingURL=../sourcemaps/boot/ApplicationBuilder.js.map
