import { Token, MetadataAdapter, MetadataExtends, ITypeDecorator, Type, LoadType } from '@ts-ioc/core';
import { AppConfigure } from '../boot/AppConfigure';
import { IApplicationBuilder } from '../boot/IApplicationBuilder';
import { IAnnotationBuilder } from '../annotations/IAnnotationBuilder';
export interface BootstrapMetadata extends AppConfigure {
    builder?: Type<IApplicationBuilder<any>> | IApplicationBuilder<any>;
    globals?: LoadType[];
}
/**
 * Bootstrap decorator, use to define class is a task element.
 *
 * @export
 * @interface IBootstrapDecorator
 * @extends {ITypeDecorator<T>}
 * @template T
 */
export interface IBootstrapDecorator<T extends BootstrapMetadata> extends ITypeDecorator<T> {
    /**
     * Bootstrap decorator, use to define class as Application Bootstrap element.
     *
     * @Bootstrap
     *
     * @param {T} metadata bootstrap metadate config.
     */
    (metadata: T): ClassDecorator;
}
/**
 * create bootstrap decorator.
 *
 * @export
 * @template T
 * @param {string} name
 * @param {(Token<IApplicationBuilder> | IApplicationBuilder)} [builder] default builder
 * @param {(Token<IAnnotationBuilder<any>> | IAnnotationBuilder<Tany>)} [annotationBuilder] default type builder.
 * @param {MetadataAdapter} [adapter]
 * @param {MetadataExtends<T>} [metadataExtends]
 * @returns {IBootstrapDecorator<T>}
 */
export declare function createBootstrapDecorator<T extends BootstrapMetadata>(name: string, builder?: Type<IApplicationBuilder<any>> | IApplicationBuilder<any>, annotationBuilder?: Token<IAnnotationBuilder<any>> | IAnnotationBuilder<any>, adapter?: MetadataAdapter, metadataExtends?: MetadataExtends<T>): IBootstrapDecorator<T>;
/**
 * Bootstrap Decorator, definde class as mvc bootstrap module.
 *
 * @Bootstrap
 */
export declare const Bootstrap: IBootstrapDecorator<BootstrapMetadata>;
