import { Type, IContainer, ModuleInjector, InjectModuleInjectorToken, IModuleValidate, Token, Providers, IModuleInjector } from '@ts-ioc/core';
import { ModuleConfigure, ModuleConfig } from './ModuleConfigure';
import { InjectedModule } from './InjectedModule';
/**
 * DIModule injector interface.
 *
 * @export
 * @interface IDIModuleInjector
 * @extends {IModuleInjector}
 */
export interface IDIModuleInjector extends IModuleInjector {
    /**
     * import module type.
     *
     * @template T
     * @param {IContainer} container
     * @param {Type<T>} type
     * @returns {Promise<InjectedModule<T>>}
     * @memberof IDIModuleInjector
     */
    import<T>(container: IContainer, type: Type<T>): Promise<InjectedModule<T>>;
    /**
     * import by config.
     *
     * @template T
     * @param {IContainer} container
     * @param {ModuleConfig<T>} config
     * @returns {Promise<any>>}
     * @memberof IDIModuleInjector
     */
    importByConfig<T>(container: IContainer, config: ModuleConfig<T>): Promise<any>;
}
/**
 * DIModule injector token.
 */
export declare const DIModuleInjectorToken: InjectModuleInjectorToken<IDIModuleInjector>;
/**
 * DIModule injector.
 *
 * @export
 * @class DIModuleInjector
 * @extends {ModuleInjector}
 */
export declare class DIModuleInjector extends ModuleInjector implements IDIModuleInjector {
    constructor(validate: IModuleValidate);
    protected setup(container: IContainer, type: Type<any>): Promise<void>;
    import<T>(container: IContainer, type: Type<T>): Promise<InjectedModule<T>>;
    importByConfig<T>(container: IContainer, config: ModuleConfig<T>): Promise<any>;
    protected importModule(container: IContainer, type: Type<any>): Promise<InjectedModule<any>>;
    protected registerConfgureDepds(container: IContainer, config: ModuleConfigure): Promise<ModuleConfigure>;
    protected importConfigExports(container: IContainer, providerContainer: IContainer, injMd: InjectedModule<any>): Promise<IContainer>;
    protected bindProvider(container: IContainer, providers: Providers[]): Token<any>[];
    static classAnnations: any;
}
