import 'reflect-metadata';
import { IContainer, Token, Providers, Registration } from '@ts-ioc/core';
import { IModuleBuilder, ModuleEnv } from './IModuleBuilder';
import { ModuleConfigure, ModuleConfig } from './ModuleConfigure';
import { ContainerPool } from '../utils';
import { IRunner, IService, Runnable } from '../runnable';
import { IAnnotationBuilder, IAnyTypeBuilder } from '../annotations';
import { InjectedModule } from './InjectedModule';
/**
 * inject module load token.
 *
 * @export
 * @class InjectModuleLoadToken
 * @extends {Registration<T>}
 * @template T
 */
export declare class InjectModuleLoadToken<T> extends Registration<T> {
    constructor(token: Token<T>);
    static classAnnations: any;
}
/**
 * module builder
 *
 * @export
 * @class ModuleBuilder
 * @implements {IModuleBuilder}
 * @template T
 */
export declare class ModuleBuilder<T> implements IModuleBuilder<T> {
    protected pools: ContainerPool;
    constructor();
    getPools(): ContainerPool;
    /**
     * build module.
     *
     * @param {(Token<T> | ModuleConfig<T>)} token
     * @param {ModuleEnv} [env]
     * @param {*} [data] bootstrap data, build data, Runnable data.
     * @returns {Promise<T>}
     * @memberof ModuleBuilder
     */
    build(token: Token<T> | ModuleConfig<T>, env?: ModuleEnv, data?: any): Promise<T>;
    /**
    * bootstrap module's main.
    *
    * @param {(Token<T> | ModuleConfig<T>)} token
    * @param {ModuleEnv} [env]
    * @param {*} [data] bootstrap data, build data, Runnable data.
    * @returns {Promise<MdInstance<T>>}
    * @memberof ModuleBuilder
    */
    bootstrap(token: Token<T> | ModuleConfig<T>, env?: ModuleEnv, data?: any): Promise<Runnable<T>>;
    import(token: Token<T>, parent?: IContainer): Promise<InjectedModule<T>>;
    protected load(token: Token<T> | ModuleConfigure, env?: ModuleEnv): Promise<InjectedModule<T>>;
    protected getParentContainer(env?: ModuleEnv): Promise<IContainer>;
    protected autoRun(container: IContainer, token: Token<any>, cfg: ModuleConfigure, instance: any, data?: any): Promise<Runnable<T>>;
    protected getDefaultRunner(container: IContainer, ...providers: Providers[]): IRunner<T>;
    protected getDefaultService(container: IContainer, ...providers: Providers[]): IService<T>;
    protected getAnnoBuilder(container: IContainer, token: Token<any>, annBuilder: Token<IAnnotationBuilder<any>> | IAnnotationBuilder<any>): IAnyTypeBuilder;
    protected getDefaultAnnBuilder(container: IContainer): IAnnotationBuilder<any>;
    /**
     * get module type
     *
     * @protected
     * @param {ModuleConfigure} cfg
     * @returns {Token<T>}
     * @memberof ModuleBuilder
     */
    protected getType(cfg: ModuleConfigure): Token<T>;
    /**
     * get boot type.
     *
     * @protected
     * @param {ModuleConfigure} cfg
     * @returns {Token<T>}
     * @memberof ModuleBuilder
     */
    protected getBootType(cfg: ModuleConfigure): Token<T>;
    static classAnnations: any;
}
