import { Registration, Token } from '@ts-ioc/core';
import { ModuleConfigure } from '../modules';
/**
 * application runer.
 *
 * @export
 * @interface IRunner
 * @template T
 */
export interface IRunner<T> {
    /**
     * run application via boot instance.
     *
     * @param {*} [data]
     * @returns {Promise<any>}
     * @memberof IRunner
     */
    run(data?: any): Promise<any>;
}
/**
 * boot element.
 *
 * @export
 * @abstract
 * @class Boot
 * @implements {IBoot}
 */
export declare abstract class Runner<T> implements IRunner<T> {
    protected token?: Token<T>;
    protected instance?: T;
    protected config?: ModuleConfigure;
    constructor(token?: Token<T>, instance?: T, config?: ModuleConfigure);
    /**
     * run boot.
     *
     * @abstract
     * @param {*} [data]
     * @returns {Promise<any>}
     * @memberof Runner
     */
    abstract run(data?: any): Promise<any>;
    static classAnnations: any;
}
/**
 * boot element
 *
 * @export
 * @class Boot
 * @extends {Runner<any>}
 */
export declare abstract class Boot extends Runner<any> {
    protected token?: Token<any>;
    protected instance?: any;
    protected config?: ModuleConfigure;
    constructor(token?: Token<any>, instance?: any, config?: ModuleConfigure);
    static classAnnations: any;
}
/**
 * application runner token.
 *
 * @export
 * @class InjectRunnerToken
 * @extends {Registration<IRunner<T>>}
 * @template T
 */
export declare class InjectRunnerToken<T> extends Registration<IRunner<T>> {
    constructor(type: Token<T>);
    static classAnnations: any;
}
/**
 * default runner token.
 */
export declare const DefaultRunnerToken: InjectRunnerToken<any>;
