import { ObjectMap } from '@ts-ioc/core';
/**
 * events
 *
 * @export
 * @interface IEvents
 */
export interface IEvents {
    /**
     * add event.
     *
     * @param {string} name
     * @param {(...args: any[]) => void} event
     * @returns {this}
     * @memberof IEvents
     */
    on(name: string, event: (...args: any[]) => void): this;
    /**
     * remove event.
     *
     * @param {string} name
     * @param {(...args: any[]) => void} [event]
     * @returns {this}
     * @memberof IEvents
     */
    off(name: string, event?: (...args: any[]) => void): this;
    /**
     * emit event.
     *
     * @param {string} name
     * @param {...any[]} args
     * @memberof IEvents
     */
    emit(name: string, ...args: any[]): void;
}
/**
 * custom events.
 *
 * @export
 * @class Events
 */
export declare class Events implements IEvents {
    eventsMap: ObjectMap<Function[]>;
    constructor();
    on(name: string, event: (...args: any[]) => void): this;
    off(name: string, event?: (...args: any[]) => void): this;
    emit(name: string, ...args: any[]): void;
    static classAnnations: any;
}
